@@ -61,14 +61,17 @@ NDK_RELEASE_REGEX = re.compile(r'r[1-9][0-9]?[a-z]?(?:-(?:rc|beta)[0-9]+)?')
 M2REPOSITORY_REVISION_REGEX = re.compile(r'android_m2repository_r([0-9]+)\.zip')
 
 # The sub-directory to install a given package into, assumes ANDROID_SDK_ROOT as root
+# https://developer.android.com/studio/command-line/
 INSTALL_DIRS = {
     'build-tools': 'build-tools/{revision}',
     'cmake': 'cmake/{revision}',
+    'cmdline-tools': 'cmdline-tools/{revision}',
     'emulator': 'emulator',
     'ndk': 'ndk/{revision}',
     'ndk-bundle': 'ndk-bundle',
     'platforms': 'platforms/{revision}',
     'platform-tools': 'platform-tools',
+    'skiaparser': 'skiaparser/{revision}',
     'tools': 'tools',
     'extras;android;m2repository': 'extras/android/m2repository',
 }
@@ -519,6 +522,7 @@ UeoNaY215eCZI12lp5pySjqSVt2rcJb1dzXp39Wk/1pN+nfJ098lT/+vlzz9XT36d/Xo/wfVo38X5v9d
 
 packages = dict()
 revisions = dict()
+platform_versions = dict()
 
 
 def verify(filename):
@@ -599,6 +603,29 @@ def parse_cmake(url, d):
             packages[key] = url
 
 
+def parse_cmdline_tools(url, d):
+    """Set up cmdline-tools with versioned and 'latest' package name"""
+    if 'source.properties' in d:
+        source_properties = get_properties_dict(d['source.properties'])
+        _add_to_revisions(url, source_properties)
+        key = tuple(source_properties['pkg.path'].split(';'))
+        if key not in packages:
+            packages[key] = url
+
+    highest = '0'
+    for key, url in packages.items():
+        if key[0] != 'cmdline-tools' or len(key) < 2:
+            continue
+        version = key[-1]
+        if version == 'latest':
+            continue
+        if LooseVersion(version) > LooseVersion(highest):
+            highest = version
+    # TODO choose version for 'latest' based on --channel
+    # https://developer.android.com/studio/releases/cmdline-tools
+    packages[('cmdline-tools', 'latest')] = packages[('cmdline-tools', highest)]
+
+
 def parse_emulator(url, d):
     if 'source.properties' in d:
         source_properties = get_properties_dict(d['source.properties'])
@@ -667,9 +694,18 @@ def parse_platforms(url, d):
         apilevel = source_properties['androidversion.apilevel']
         # TODO this should make all versions/revisions available, not only most recent
         key = ('platforms', 'android-%s' % apilevel)
-        if re.match(r'^[1-9].*', source_properties.get('platform.version', '')):
+        vstring = '%s.%s' % (
+            source_properties.get('platform.version'),
+            source_properties.get('pkg.revision'),
+        )
+        if re.match(r'^[1-9].*', vstring):
+            if key not in platform_versions:
+                platform_versions[key] = []
+            platform_version = LooseVersion(vstring)
+            platform_versions[key].append(platform_version)
             if key in packages:
-                packages[key] = sorted([url, packages.get(key)])[-1]
+                if platform_version == sorted(platform_versions[key])[-1]:
+                    packages[key] = url
             else:
                 packages[key] = url
 
@@ -715,6 +751,18 @@ def parse_tools(url, d):
     packages[('tools',)] = packages[('tools', highest)]
 
 
+def parse_skiaparser(url, d):
+    """Set up skiaparser with versioned and unversioned package name"""
+    if 'source.properties' in d:
+        source_properties = get_properties_dict(d['source.properties'])
+        _add_to_revisions(url, source_properties)
+        key = tuple(source_properties['pkg.path'].split(';'))
+        if key in packages:
+            packages[key] = sorted([url, packages[key]])[-1]
+        else:
+            packages[key] = url
+
+
 def parse_repositories_cfg(f):
     """parse the supplied repositories.cfg and return a list of URLs"""
     with Path(f).open() as fp:
@@ -801,6 +849,11 @@ def _process_checksums(checksums):
         elif basename.startswith('cmake'):
             for entry in checksums[url]:
                 parse_cmake(url, entry)
+        elif basename.startswith('cmdline-tools') or basename.startswith(
+            'commandlinetools'
+        ):
+            for entry in checksums[url]:
+                parse_cmdline_tools(url, entry)
         elif basename.startswith('emulator'):
             for entry in checksums[url]:
                 parse_emulator(url, entry)
@@ -815,6 +868,9 @@ def _process_checksums(checksums):
         elif basename.startswith('android-') or basename.startswith('platform-'):
             for entry in checksums[url]:
                 parse_platforms(url, entry)
+        elif basename.startswith('skiaparser'):
+            for entry in checksums[url]:
+                parse_skiaparser(url, entry)
         elif basename.startswith('tools') or basename.startswith('sdk-tools-'):
             for entry in checksums[url]:
                 parse_tools(url, entry)
